use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
AUTOHANDLERS
Admin
Adminstrator
Alex
Apache
ApacheHandler
ApacheModPerl
ApacheReload
Arnfjörð
Autohandlers
Balhatchet
BaseCache
Bekman
Bjarmason
CGI
CGIHandler
Cache
Compiler
Component
ComponentSource
ContactUs
Context
DROLSKY
DROLSKY's
DSO
Dave
DeWitt
Devel
DocumentRoot
DocumentRoots
Escapes
Exceptions
FAQ
FakeApache
Falcone
FastCGI
File
FileBased
FilesMatch
Fish
Florian
Follett
ForceFileDownload
Fredric
GIF
Georgiou
HPUX
HTML
HUP
Handler
HandlingDirectoriesWithDhandlers
Interp
John
Jonathan
Kane
Ken
Kent
Kevin
Khera
Kiriakos
Kirwan
Kumar
LFU
Lexer
LogLevel
MSIE
MailingLists
Mallah
Mason
MasonAllowGlobals
MasonApacheStatusTitle
MasonArgsMethod
MasonAutoSendHeaders
MasonAutoflush
MasonAutohandlerName
MasonBufferPreallocateSize
MasonCodeCacheMaxSize
MasonCompClass
MasonCompRoot
MasonCompilerClass
MasonComponentErrorHandler
MasonDataCacheApi
MasonDataCacheDefaults
MasonDataDir
MasonDeclineDirs
MasonDefaultEscapeFlags
MasonDefineArgsHash
MasonDhandlerName
MasonDynamicCompRoot
MasonEnableAutoflush
MasonErrorFormat
MasonErrorMode
MasonEscapeFlags
MasonIgnoreWarningsExpr
MasonInPackage
MasonInterpClass
MasonLexerClass
MasonMaxRecurse
MasonNamedComponentSubs
MasonObjectFileExtension
MasonOutMethod
MasonPlugins
MasonPostamble
MasonPostprocessPerl
MasonPostprocessText
MasonPreamble
MasonPreloads
MasonPreprocess
MasonRequestClass
MasonResolverClass
MasonStaticSource
MasonStaticSourceTouchFile
MasonSubcompClass
MasonUseObjectFiles
MasonUseSourceLineNumbers
MasonUseStrict
MasonUseWarnings
MethodMaker
Null
NullCache
O'Reilly
PRs
Params
Parser
Patrick
PayPal
PerlFreshRestart
PerlHandler
PerlModule
PerlSetVar
Plugin
Preallocating
Preloading
RPMs
Rajesh
RedHat
ReloadAll
Request
Resolver
Ricardo
Rolsky
Rolsky's
SUBCLASSABLE
SYNOPIS
Schlichting
Shlomi
Signes
Solaris
SpeedyCGI
Stas
Subclassing
Subcomponent
Subcomponents
Subrequests
Swartz
TIEHASH
Tests
ToObject
Tools
USR
UserDir
Utils
Vandiver
Vivek
Williams
ala
alex
apachectl
apachehandler
attr
autarch
autohandler
autohandlers
avarab
bgcolor
bin
breakpoint'able
certian
checksum
conf
convert0
corrup
defined'ness
dhandler
dhandlers
drolsky
dynamicImage
falcone
faq
fh
fido
filenaming
foobarbaz
fsfs
gif
gifs
htaccess
html
interp
isNetscape
ized
izing
jpegs
jwilliams
kaoru
ken
kentnl
lexed
lib
libapreq
libexpat
mason
mc
mcomp
mhtml
modus
mpl
mtxt
nh
onwards
optimizations
overrideable
perlsub
postprocess
predeclaring
preload
preloaded
preloading
preloads
prepopulate
preprocess
profiler
rdist
reallocations
reparsed
reuseability
rjbs
scomp
se
serializable
shlomif
sql
srm
subcomponent
subcomponents
subcomps
subexec
subrequest
subrequests
swartz
taglibs
tgz
tmp
todo
un
undeclarable
unweakened
updateable
uring
use'd
xml
Ævar
